{--------------------------------------------------------------------------
Sprache    : Delphi 1.0
Aufgaben   : Funktionen und Prozeduren fuer Pascal-Strings
Hinweise   : Programme, die diese Unit benutzen, muessen mit der
             Anweisung $X+ compiliert werden
Fehler     : Fehler bitte melden an: scheffczyk@verwaltung.uni-mainz.de
             GetToken     = Gibt einen bestimmten Teilstring zurueck, der ueber
                            seine Position (Teilstring Nr n) bestimmt wird
             GetTNum      = Gibt die Anzahl der gefundenen Teilstrings zurueck,
                            die durch best. Zeichen voneinander getrennt sind
             GetTSep      = Gibt das Trennzeichen zurueck, das zwei bestimmte
                            Teilstrings voneinander trennt
---------------------------------------------------------------------------}
unit _4token;

interface

{--------------------------------------------------------------------------
Name      : GetTNum
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt die Anzahl von Substrings in einem String zurck die durch
            ein Zeichen aus dem Seperatorstring voneinander getrennt sind
Parameter : SourceStr : String : Der zu durchsuchende String
            StringSep : String : String mit den Seperatorzeichen
Rueckgabe :             Byte   : Anzahl der gefundenen Substrings
Fehler    :
Hinweise  : Wenn SourceStr ein Leerstring ist wird 0 zurueckgegeben
---------------------------------------------------------------------------}
function GetTNum( SourceStr,
                  SepChars : String): Byte;

{--------------------------------------------------------------------------
Name      : GetToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt einen Substring aus einem String zurueck, wobei Substrings
            durch ein Zeichen aus dem Seperatorstring voneinander getrennt
            sind
Parameter : SourceStr : String : Der zu durchsuchende String
            StringSep : String : String mit den Seperatorzeichen
            TokenNum  : Byte   : Nummer des gewuenschten Substrings
Rueckgabe :             String : Der sparierte Substring
Fehler    :
Hinweise  :
---------------------------------------------------------------------------}
function GetToken( SourceStr,
                   StringSep : String;
                   TokenNum  : Byte) : String;

{--------------------------------------------------------------------------
Name      : GetTSep
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt das Separatorzeichen zurueck, dass VOR dem angegebenen
            Substring liegt
Parameter : SourceStr : String : Der zu durchsuchende String
            StringSep : String : Zeiger auf den Seperatorzeichen-String
            TokenNum  : Byte   : Nummer des gewuenschten Substrings
Rueckgabe :             Char   : Das Separatorzeichen
Fehler    :
Hinweise  : Wenn der angegebene Substring nicht existiert oder ein Leerstring
            uebergeben wurde, wird das ASCII-Zeichen 0 zurueckgegeben
---------------------------------------------------------------------------}
function GetTSep(  SourceStr,
                   StringSep : String;
                   TokenNum  : Byte) : String;

{--------------------------------------------------------------------------
Name      : StrGetTNum
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt die Anzahl von Substrings in einem String zurck die durch
            ein Zeichen aus dem Separatorstring voneinander getrennt sind
Parameter : SourceStr : PChar : Zeiger auf den zu durchsuchenden String
            StringSep : PChar : Zeiger auf den Seperatorzeichen-String
Rueckgabe :             Word  : Anzahl der gefundenen Substrings
Fehler    :
Hinweise  : Wenn SourceStr ein Leerstring oder NIL ist wird 0 zurueckgegeben
---------------------------------------------------------------------------}
function StrGetTNum( SourceStr,
                     StringSep : PChar): Word;

{--------------------------------------------------------------------------
Name      : StrGetToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt einen Substring aus einem String zurueck, wobei Substrings
            durch ein Zeichen aus dem Seperatorstring voneinander getrennt
            sind
Parameter : SourceStr : PChar : Zeiger auf den zu durchsuchenden String
            StringSep : PChar : Zeiger auf den Seperatorzeichen-String
            theToken  : PChar : Zeiger auf den Heap-Bereich zum Abspeichern
                                des Substrings
            TokenNum  : Word  : Nummer des gewuenschten Substrings
Rueckgabe :             PChar : Zeiger auf den separierten Substring
                                (entsprichz 'theToken', siehe Hinweise)
Fehler    :
Hinweise  : Der Parameter 'theToken' wird uebergeben, damit die Verantwortung
            sowohl fuer die Erstellung, als auch fuer die Freigabe des
            Heapbereiches fuer den Substring bei dem aufrufenden Routine liegt
---------------------------------------------------------------------------}
function StrGetToken( SourceStr,
                      StringSep    : PChar;
                      var theToken : PChar;
                      TokenNum     : Word) : PChar;

{--------------------------------------------------------------------------
Name      : StrGetTSep
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 24.08.1995
Autor     : Scheffczyk
Aufgabe   : Gibt das Separatorzeichen zurueck, dass VOR dem angegebenen
            Substring liegt
Parameter : SourceStr : PChar : Zeiger auf den zu durchsuchenden String
            StringSep : PChar : Zeiger auf den Seperatorzeichen-String
            TokenNum  : Word  : Nummer des gewuenschten Substrings
Rueckgabe :             Char  : Das Separatorzeichen
Fehler    :
Hinweise  : Wenn der angegebene Substring nicht existiert oder ein Leerstring
            uebergeben wurde, wird das ASCII-Zeichen 0 zurueckgegeben
---------------------------------------------------------------------------}
function StrGetTSep( SourceStr,
                     StringSep : PChar;
                     TokenNum  : Word): Char;


implementation

uses
    SysUtils;

{--------------------------------------------------------------------------}
function StrCharIsNotInArray(RedChars: PChar; aChar : Char): Boolean;
var
   aRChar  : PChar;
begin
     aRChar  := RedChars;
     while ((aRChar^ <> aChar) and (aRChar^ <> #0)) do
     begin
          inc(aRChar);
     end;
     if aRChar^ = #0 then
     begin
          Result := true;
     end
     else
     begin
          Result := false;
     end;
end;

{--------------------------------------------------------------------------}
function StrGetTNum( SourceStr,
                     StringSep : PChar): Word;
var
   TokensFound : Word;
   SourcePos   : PChar;
begin
     TokensFound := 0;
     if StrLen(SourceStr) > 0 then
     begin
          Inc(TokensFound);
          SourcePos := SourceStr;
          while SourcePos^ <> #0 do
          begin
               if not StrCharIsNotInArray(StringSep,SourcePos^) then
               begin
                    Inc(TokensFound);
               end;
               Inc(SourcePos);
          end;
     end;
     Result := TokensFound;
end;
{--------------------------------------------------------------------------}
function StrGetToken( SourceStr,
                      StringSep    : PChar;
                      var theToken : PChar;
                      TokenNum  : Word) : PChar;
var
   TokensFound : Word;
   SourcePos   : PChar;
   TargetPos   : PChar;
begin
     TokensFound := 0;
     if StrLen(SourceStr) > 0 then
     begin
          Inc(TokensFound);
          SourcePos := SourceStr;
          TargetPos := theToken;
          while ((TokensFound <= TokenNum) and (SourcePos^ <> #0)) do
          begin
               if not StrCharIsNotInArray(StringSep,SourcePos^) then
               begin
                    Inc(TokensFound);
               end
               else
               begin
                    if TokensFound = TokenNum then
                    begin
                         TargetPos^ := SourcePos^;
                         Inc(TargetPos);
                    end;
               end;
               Inc(SourcePos);
          end;
          TargetPos^ := #0;
     end
     else
     begin
          theToken^ := #0
     end;
     Result := theToken;
end;

{--------------------------------------------------------------------------}
function StrGetTSep( SourceStr,
                     StringSep : PChar;
                     TokenNum  : Word) : Char;
var
   TokensFound : Word;
   SourcePos   : PChar;
   SepChar     : Char;
begin
     TokensFound := 0;
     if StrLen(SourceStr) > 0 then
     begin
          Inc(TokensFound);
          SourcePos := SourceStr;
          SepChar   := #0;
          while ((TokensFound <= TokenNum+1) and (SourcePos^ <> #0)) do
          begin
               if not StrCharIsNotInArray(StringSep,SourcePos^) then
               begin
                    Inc(TokensFound);
                    if TokensFound = TokenNum+1 then
                    begin
                         SepChar := SourcePos^;
                    end;
               end;
               Inc(SourcePos);
          end;
     end
     else
     begin
          SepChar := #0
     end;
     Result := SepChar;
end;

{--------------------------------------------------------------------------}
function GetTNum( SourceStr : String;
                  SepChars  : String): Byte;
var
   aPChar1  : PChar;
   aPChar2  : PChar;
begin
     aPChar1 := StrAlloc(Length(SourceStr)+1);
     aPChar2 := StrAlloc(Length(SepChars)+1);
     StrPCopy(aPChar1,SourceStr);
     StrPCopy(aPChar2,SepChars);
     Result  := StrGetTNum(aPChar1,aPChar2);
     StrDispose(aPChar1);
     StrDispose(aPChar2);
end;

{--------------------------------------------------------------------------}
function GetToken( SourceStr,
                   StringSep : String;
                   TokenNum  : Byte) : String;
var
   aPChar1  : PChar;
   aPChar2  : PChar;
   newPChar : PChar;
begin
     aPChar1   := StrAlloc(Length(SourceStr)+1);
     aPChar2   := StrAlloc(Length(StringSep)+1);
     newPChar  := StrAlloc(Length(SourceStr)+1);
     StrPCopy(aPChar1,SourceStr);
     StrPCopy(aPChar2,StringSep);
     newPChar := StrGetToken(aPChar1,aPChar2,newPChar,TokenNum);
     Result   := StrPas(newPChar);
     StrDispose(aPChar1);
     StrDispose(aPChar2);
     StrDispose(newPChar);
end;


{--------------------------------------------------------------------------}
function GetTSep(  SourceStr,
                   StringSep : String;
                   TokenNum  : Byte) : String;
var
   aPChar1  : PChar;
   aPChar2  : PChar;
begin
     aPChar1   := StrAlloc(Length(SourceStr)+1);
     aPChar2   := StrAlloc(Length(StringSep)+1);
     StrPCopy(aPChar1,SourceStr);
     StrPCopy(aPChar2,StringSep);
     Result   := StrGetTSep(aPChar1,aPChar2,TokenNum);
     StrDispose(aPChar1);
     StrDispose(aPChar2);
end;

{==========================================================================}
initialization

end.
